from . import _
from enigma import ePythonMessagePump
from __init__ import decrypt_block
from ThreadQueue import ThreadQueue
import gdata.youtube
import gdata.youtube.service
from gdata.service import BadAuthentication
from twisted.web import client
from twisted.internet import reactor
from urllib2 import Request, URLError, urlopen as urlopen2
from socket import gaierror, error
import os, socket, httplib
from urllib import quote, unquote_plus, unquote, urlencode
from httplib import HTTPConnection, CannotSendRequest, BadStatusLine, HTTPException
from Components.config import config
from urlparse import parse_qs, parse_qsl
from threading import Thread
import re
import json
HTTPConnection.debuglevel = 1
if 'HTTPSConnection' not in dir(httplib):
    gdata.youtube.service.YOUTUBE_USER_FEED_URI = 'http://gdata.youtube.com/feeds/api/users'

def validate_cert(cert, key):
    buf = decrypt_block(cert[8:], key)
    if buf is None:
        return
    else:
        return buf[36:107] + cert[139:196]


def get_rnd():
    try:
        rnd = os.urandom(8)
        return rnd
    except:
        return None

    return None


std_headers = {'User-Agent': 'Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/20100627 Firefox/3.6.6',
 'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
 'Accept-Language': 'en-us,en;q=0.5'}

class GoogleSuggestions():

    def __init__(self):
        self.hl = 'en'
        self.conn = None
        return

    def prepareQuery(self):
        self.prepQuerry = '/complete/search?output=toolbar&client=youtube&xml=true&ds=yt&'
        if self.hl is not None:
            self.prepQuerry = self.prepQuerry + 'hl=' + self.hl + '&'
        self.prepQuerry = self.prepQuerry + 'jsonp=self.gotSuggestions&q='
        print '[MyTube - GoogleSuggestions] prepareQuery:', self.prepQuerry
        return

    def getSuggestions(self, queryString):
        self.prepareQuery()
        if queryString is not '':
            query = self.prepQuerry + quote(queryString)
            self.conn = HTTPConnection('google.com')
            try:
                self.conn = HTTPConnection('google.com')
                self.conn.request('GET', query, '', {'Accept-Encoding': 'UTF-8'})
            except (CannotSendRequest, gaierror, error):
                self.conn.close()
                print '[MyTube - GoogleSuggestions] Can not send request for suggestions'
                return None

            try:
                response = self.conn.getresponse()
            except BadStatusLine:
                self.conn.close()
                print '[MyTube - GoogleSuggestions] Can not get a response from google'
                return None

            if response.status == 200:
                data = response.read()
                header = response.getheader('Content-Type', 'text/xml; charset=ISO-8859-1')
                charset = 'ISO-8859-1'
                try:
                    charset = header.split(';')[1].split('=')[1]
                    print '[MyTube - GoogleSuggestions] Got charset %s' % charset
                except:
                    print '[MyTube - GoogleSuggestions] No charset in Header, falling back to %s' % charset

                data = data.decode(charset).encode('utf-8')
                self.conn.close()
                return data
            else:
                self.conn.close()
                return None
        else:
            return None
        return None


class MyTubeFeedEntry():

    def __init__(self, feed, entry, favoritesFeed = False):
        self.feed = feed
        self.entry = entry
        self.favoritesFeed = favoritesFeed
        self.thumbnail = {}

    def isPlaylistEntry(self):
        return False

    def getTubeId(self):
        ret = None
        if self.entry.media.player:
            split = self.entry.media.player.url.split('=')
            ret = split.pop()
            if ret.startswith('youtube_gdata'):
                tmpval = split.pop()
                if tmpval.endswith('&feature'):
                    tmp = tmpval.split('&')
                    ret = tmp.pop(0)
        return ret

    def getTitle(self):
        return self.entry.media.title.text

    def getDescription(self):
        if self.entry.media is not None and self.entry.media.description is not None:
            return self.entry.media.description.text
        else:
            return 'not vailable'

    def getThumbnailUrl(self, index = 0):
        if index < len(self.entry.media.thumbnail):
            return self.entry.media.thumbnail[index].url
        else:
            return None

    def getPublishedDate(self):
        if self.entry.published is not None:
            return self.entry.published.text
        else:
            return 'unknown'

    def getViews(self):
        if self.entry.statistics is not None:
            return self.entry.statistics.view_count
        else:
            return 'not available'

    def getDuration(self):
        if self.entry.media is not None and self.entry.media.duration is not None:
            return self.entry.media.duration.seconds
        else:
            return 0
            return

    def getRatingAverage(self):
        if self.entry.rating is not None:
            return self.entry.rating.average
        else:
            return 0

    def getNumRaters(self):
        if self.entry.rating is not None:
            return self.entry.rating.num_raters
        else:
            return ''

    def getAuthor(self):
        authors = []
        for author in self.entry.author:
            authors.append(author.name.text)

        author = ', '.join(authors)
        return author

    def getUserFeedsUrl(self):
        for author in self.entry.author:
            return author.uri.text

        return False

    def getUserId(self):
        return self.getUserFeedsUrl().split('/')[-1]

    def subscribeToUser(self):
        username = self.getUserId()
        return myTubeService.SubscribeToUser(username)

    def addToFavorites(self):
        video_id = self.getTubeId()
        return myTubeService.addToFavorites(video_id)

    def PrintEntryDetails(self):
        EntryDetails = {'Title': None,
         'TubeID': None,
         'Published': None,
         'Published': None,
         'Description': None,
         'Category': None,
         'Tags': None,
         'Duration': None,
         'Views': None,
         'Rating': None,
         'Thumbnails': None}
        EntryDetails['Title'] = self.entry.media.title.text
        EntryDetails['TubeID'] = self.getTubeId()
        EntryDetails['Description'] = self.getDescription()
        EntryDetails['Category'] = self.entry.media.category[0].text
        EntryDetails['Tags'] = self.entry.media.keywords.text
        EntryDetails['Published'] = self.getPublishedDate()
        EntryDetails['Views'] = self.getViews()
        EntryDetails['Duration'] = self.getDuration()
        EntryDetails['Rating'] = self.getNumRaters()
        EntryDetails['RatingAverage'] = self.getRatingAverage()
        EntryDetails['Author'] = self.getAuthor()
        list = []
        for thumbnail in self.entry.media.thumbnail:
            print 'Thumbnail url: %s' % thumbnail.url
            list.append(str(thumbnail.url))

        EntryDetails['Thumbnails'] = list
        return EntryDetails

    def getPage(self, url):
        watchvideopage = None
        watchrequest = Request(url, None, std_headers)
        try:
            print '[MyTube] trying to find out if a HD Stream is available', url
            watchvideopage = urlopen2(watchrequest).read()
        except (URLError, HTTPException, socket.error) as err:
            print '[MyTube] Error: Unable to retrieve watchpage - Error code: ', str(err)
            return

        return watchvideopage

    def getVideoUrl(self):
        VIDEO_FMT_PRIORITY_MAP = {'38': 1,
         '37': 2,
         '22': 3,
         '18': 4,
         '35': 5,
         '34': 6}
        video_url = None
        video_id = str(self.getTubeId())
        watch_url = 'http://www.youtube.com/watch?v=%s&gl=US&hl=en' % video_id
        for x in range(0, 10):
            watchvideopage = self.getPage(watch_url)
            if watchvideopage != None:
                break

        if watchvideopage == None:
            return video_url
        else:
            age_gate = False
            if re.search('player-age-gate-content">', watchvideopage) is not None:
                age_gate = True
                if config.plugins.mytube.general.showadult.value == False:
                    return 'age'
                info_url = 'http://www.youtube.com/get_video_info?video_id=%s&el=embedded&gl=US&hl=en&eurl=%s&asv=3&sts=1588' % (video_id, 'https://youtube.googleapis.com/v/' + video_id)
                request = Request(info_url, None, std_headers)
                try:
                    infopage = urlopen2(request).read()
                    videoinfo = parse_qs(infopage)
                except (URLError, HTTPException, socket.error) as err:
                    print '[MyTube] Error: unable to download video infopage', str(err)
                    return video_url

            else:
                videoinfo = None
                for el in ['',
                 '&el=detailpage',
                 '&el=vevo',
                 '&el=embedded']:
                    info_url = 'http://www.youtube.com/get_video_info?&video_id=%s%s&ps=default&eurl=&gl=US&hl=en' % (video_id, el)
                    for x in range(0, 10):
                        infopage = self.getPage(info_url)
                        if infopage != None:
                            break

                    if infopage == None:
                        return video_url
                    try:
                        videoinfo = parse_qs(infopage)
                        if ('url_encoded_fmt_stream_map' or 'fmt_url_map') in videoinfo:
                            break
                    except:
                        pass

                if videoinfo == None:
                    print '[MyTube] Error: unable to download video infopage'
                    return video_url
            try:
                mobj = re.search(';ytplayer.config = ({.*?});', watchvideopage)
                info = json.loads(mobj.group(1))
                args = info['args']
                if args.get('ptk', '') == 'vevo' or '&s=' in str(args):
                    print '[MyTube] %s: Vevo video detected with encrypted signature.' % video_id
                    videoinfo['url_encoded_fmt_stream_map'] = [str(args['url_encoded_fmt_stream_map'])]
            except:
                pass

            jsversion = None
            try:
                jsversion = re.compile('html5player-([^.]+).js', re.DOTALL).search(watchvideopage).group(1).replace('\\', '')
            except:
                pass

            if ('url_encoded_fmt_stream_map' or 'fmt_url_map') not in videoinfo:
                if 'reason' not in videoinfo:
                    print '[MyTube] Error: unable to extract "fmt_url_map" or "url_encoded_fmt_stream_map" parameter for unknown reason'
                else:
                    reason = unquote_plus(videoinfo['reason'][0])
                    print '[MyTube] Error: YouTube said: %s' % reason.decode('utf-8')
                return video_url
            video_fmt_map = {}
            fmt_infomap = {}
            if videoinfo.has_key('url_encoded_fmt_stream_map'):
                tmp_fmtUrlDATA = videoinfo['url_encoded_fmt_stream_map'][0].split(',')
            else:
                tmp_fmtUrlDATA = videoinfo['fmt_url_map'][0].split(',')
            for fmtstring in tmp_fmtUrlDATA:
                fmturl = fmtid = fmtsig = ''
                if videoinfo.has_key('url_encoded_fmt_stream_map'):
                    try:
                        for arg in fmtstring.split('&'):
                            if arg.find('=') >= 0:
                                print arg.split('=')
                                key, value = arg.split('=')
                                if key == 'itag':
                                    if len(value) > 3:
                                        value = value[:2]
                                    fmtid = value
                                elif key == 'url':
                                    fmturl = value
                                elif key == 'sig':
                                    fmtsig = value
                                elif key == 's':
                                    if age_gate:
                                        fmtsig = self._decrypt_signature_age_gate(value)
                                    else:
                                        fmtsig = self._decrypt_signature(value, jsversion)

                        if fmtid != '' and fmturl != '' and VIDEO_FMT_PRIORITY_MAP.has_key(fmtid):
                            video_fmt_map[VIDEO_FMT_PRIORITY_MAP[fmtid]] = {'fmtid': fmtid,
                             'fmturl': unquote_plus(fmturl),
                             'fmtsig': fmtsig}
                            if fmtsig != '':
                                fmt_infomap[int(fmtid)] = '%s&signature=%s' % (unquote_plus(fmturl), fmtsig)
                            elif '%26signature%3D' in fmturl:
                                fmt_infomap[int(fmtid)] = unquote_plus(fmturl)
                        fmturl = fmtid = fmtsig = ''
                    except:
                        print 'error parsing fmtstring:', fmtstring

                else:
                    fmtid, fmturl = fmtstring.split('|')
                    if VIDEO_FMT_PRIORITY_MAP.has_key(fmtid) and fmtid != '':
                        video_fmt_map[VIDEO_FMT_PRIORITY_MAP[fmtid]] = {'fmtid': fmtid,
                         'fmturl': unquote_plus(fmturl)}
                        fmt_infomap[int(fmtid)] = unquote_plus(fmturl)

            print '[MyTube] got', sorted(fmt_infomap.iterkeys())
            if video_fmt_map and len(video_fmt_map):
                print '[MyTube] found best available video format:', video_fmt_map[sorted(video_fmt_map.iterkeys())[0]]['fmtid']
                best_video = video_fmt_map[sorted(video_fmt_map.iterkeys())[0]]
                video_url = '%s&signature=%s' % (best_video['fmturl'].split(';')[0], best_video['fmtsig'])
                print '[MyTube] found best available video url:', video_url
            return video_url

    def _decrypt_signature2(self, s):
        if len(s) == 92:
            return s[25] + s[3:25] + s[0] + s[26:42] + s[79] + s[43:79] + s[91] + s[80:83]
        elif len(s) == 90:
            return s[25] + s[3:25] + s[2] + s[26:40] + s[77] + s[41:77] + s[89] + s[78:81]
        elif len(s) == 89:
            return s[84:78:-1] + s[87] + s[77:60:-1] + s[0] + s[59:3:-1]
        elif len(s) == 88:
            return s[48] + s[81:67:-1] + s[82] + s[66:62:-1] + s[85] + s[61:48:-1] + s[67] + s[47:12:-1] + s[3] + s[11:3:-1] + s[2] + s[12]
        elif len(s) == 87:
            return s[83:53:-1] + s[3] + s[52:40:-1] + s[86] + s[39:10:-1] + s[0] + s[9:3:-1] + s[53]
        elif len(s) == 86:
            return s[5:20] + s[2] + s[21:]
        elif len(s) == 85:
            return s[2:8] + s[0] + s[9:21] + s[65] + s[22:65] + s[84] + s[66:82] + s[21]
        elif len(s) == 84:
            return s[83:27:-1] + s[0] + s[26:5:-1] + s[2:0:-1] + s[27]
        elif len(s) == 83:
            return s[:15] + s[80] + s[16:80] + s[15]
        elif len(s) == 82:
            return s[36] + s[79:67:-1] + s[81] + s[66:40:-1] + s[33] + s[39:36:-1] + s[40] + s[35] + s[0] + s[67] + s[32:0:-1] + s[34]
        elif len(s) == 81:
            return s[56] + s[79:56:-1] + s[41] + s[55:41:-1] + s[80] + s[40:34:-1] + s[0] + s[33:29:-1] + s[34] + s[28:9:-1] + s[29] + s[8:0:-1] + s[9]
        elif len(s) == 79:
            return s[54] + s[77:54:-1] + s[39] + s[53:39:-1] + s[78] + s[38:34:-1] + s[0] + s[33:29:-1] + s[34] + s[28:9:-1] + s[29] + s[8:0:-1] + s[9]
        else:
            print '[MyTube] Unable to decrypt signature, length not valid'
            return ''

    def _decrypt_signature_age_gate(self, s):
        if len(s) == 86:
            return s[2:63] + s[82] + s[64:82] + s[63]
        else:
            return self._decrypt_signature2(s)

    def _decrypt_signature(self, s, js):
        cyphers = {'vflNzKG7n': 's3 r s2 r s1 r w67',
         'vfllMCQWM': 's2 w46 r w27 s2 w43 s2 r',
         'vflJv8FA8': 's1 w51 w52 r',
         'vflR_cX32': 's2 w64 s3',
         'vflveGye9': 'w21 w3 s1 r w44 w36 r w41 s1',
         'vflj7Fxxt': 'r s3 w3 r w17 r w41 r s2',
         'vfltM3odl': 'w60 s1 w49 r s1 w7 r s2 r',
         'vflDG7-a-': 'w52 r s3 w21 r s3 r',
         'vfl39KBj1': 'w52 r s3 w21 r s3 r',
         'vflmOfVEX': 'w52 r s3 w21 r s3 r',
         'vflJwJuHJ': 'r s3 w19 r s2',
         'vfl_ymO4Z': 'r s3 w19 r s2',
         'vfl26ng3K': 'r s2 r',
         'vflcaqGO8': 'w24 w53 s2 w31 w4',
         'vflQw-fB4': 's2 r s3 w9 s3 w43 s3 r w23',
         'vflSAFCP9': 'r s2 w17 w61 r s1 w7 s1',
         'vflART1Nf': 's3 r w63 s2 r s1',
         'vflLC8JvQ': 'w34 w29 w9 r w39 w24',
         'vflm_D8eE': 's2 r w39 w55 w49 s3 w56 w2',
         'vflTWC9KW': 'r s2 w65 r',
         'vflRFcHMl': 's3 w24 r',
         'vflM2EmfJ': 'w10 r s1 w45 s2 r s3 w50 r',
         'vflz8giW0': 's2 w18 s3',
         'vfl_wGgYV': 'w60 s1 r s1 w9 s3 r s3 r',
         'vfl1HXdPb': 'w52 r w18 r s1 w44 w51 r s1',
         'vfl2LOvBh': 'w34 w19 r s1 r s3 w24 r',
         'vflZK4ZYR': 'w19 w68 s1',
         'vflh9ybst': 'w48 s3 w37 s2',
         'vflg0g8PQ': 'w36 s3 r s2',
         'vflg0g8PQ': 'w36 s3 r s2',
         'vflHOr_nV': 'w58 r w50 s1 r s1 r w11 s3',
         'vfluy6kdb': 'r w12 w32 r w34 s3 w35 w42 s2',
         'vflkuzxcs': 'w22 w43 s3 r s1 w43',
         'vflGNjMhJ': 'w43 w2 w54 r w8 s1',
         'vfldJ8xgI': 'w11 r w29 s1 r s3'}
        if js == None:
            return ''
        else:
            if js in cyphers:
                arr = cyphers[js]
            else:
                arr = self.guessjs(js)
            return self.decode(s, arr)

    def decode(self, sig, arr):
        sigA = sig
        if arr == '':
            return ''
        arr2 = arr.split(' ')
        for act in arr2:
            if act[0] == 'w':
                sigA = self.swap(sigA, act[1:])
            elif act == 'r':
                sigA = sigA[::-1]
            else:
                act2 = int(act[1:])
                sigA = sigA[act2:]

        result = sigA
        if len(result) == 81:
            return result
        else:
            return ''

    def swap(self, a, b):
        al = list(a)
        b = int(b) % len(a)
        c = al[0]
        al[0] = al[int(b)]
        al[int(b)] = c
        return ''.join(al)

    def guessjs(self, ver):
        import plugin
        cypher = ''
        try:
            f = open(plugin.plugin_path + '/mytube.dic', 'r')
            for line in f:
                if ver in line:
                    cypher = line.split(':')[1]
                    break

            f.close()
        except:
            pass

        if cypher != '':
            return cypher
        url = 'http://s.ytimg.com/yts/jsbin/html5player-%s.js' % ver
        try:
            script = urlopen2(url).read()
        except:
            print '[myTube] unable to download javascript code'
            return ''

        if script:
            try:
                nameFunct = re.search('.signature=(.*?)\\(', script).group(1)
            except:
                print '[myTube] unable to find decode name function'
                return ''

            try:
                Funct = re.search('function %s[^{]+{(.*?)}' % nameFunct.replace('$', '\\$'), script).group(1)
                print '[myTube] Function for decrypter is %s' % Funct
            except:
                print '[myTube] unable to find decode function %s' % nameFunct
                return ''

            try:
                for f in Funct.split(';'):
                    if 'split' not in f and 'join' not in f and 'b=' not in f and '=b' not in f:
                        if 'reverse' in f:
                            cypher = cypher + 'r '
                        elif 'a.length' in f:
                            par = f.split('%')[0].split('=')[1].split('[')[-1]
                            cypher = cypher + 'w%s ' % par
                        elif '.' in f and '=' not in f:
                            par = re.search('\\((.*?)\\)', f).group(1)
                            par = par.split(',')[1]
                            cypher = cypher + self.getfunction(f, script, par)
                        else:
                            par = re.search('\\((.*?)\\)', f).group(1)
                            if len(par.split(',')) == 2:
                                par = par.split(',')[1]
                                cypher = cypher + 'w%s ' % par
                            else:
                                cypher = cypher + 's%s ' % par

            except:
                print '[myTube] unable to make cypher string for decrypt'
                return ''

            cypher = cypher.rstrip()
            import datetime
            open(plugin.plugin_path + '/mytube.dic', 'a').write('%s:%s:%s\n' % (ver, cypher, datetime.datetime.now().strftime('%d %b %Y')))
            print '[myTube] Decode cypher is %s' % cypher
            return cypher

    def getfunction(self, func, script, par):
        cypher = ''
        nfunc = func.split('(')[0]
        nfunc = nfunc.split('.')[1]
        try:
            Funct = re.search('%s:function[^{]+{(.*?)}' % nfunc.replace('$', '\\$'), script).group(1)
            print '[myTube] SubFunction for decrypter is %s' % Funct
        except:
            print '[myTube] unable to find decode subfunction %s' % nfunc
            return ''

        try:
            if 'reverse' in Funct:
                cypher = 'r '
            elif 'a.length' in Funct:
                cypher = 'w%s ' % par
            elif 'splice' in Funct:
                cypher = 's%s ' % par
            elif len(par.split(',')) == 2:
                par = par.split(',')[1]
                cypher = 'w%s ' % par
            else:
                cypher = 's%s ' % par
        except:
            print '[myTube] unable to make cypher string for decrypt'
            return ''

        return cypher

    def getRelatedVideos(self):
        print '[MyTubeFeedEntry] getRelatedVideos()'
        for link in self.entry.link:
            if link.rel.endswith('video.related'):
                print 'Found Related: ', link.href
                return link.href

    def getResponseVideos(self):
        print '[MyTubeFeedEntry] getResponseVideos()'
        for link in self.entry.link:
            if link.rel.endswith('video.responses'):
                print 'Found Responses: ', link.href
                return link.href

    def getUserVideos(self):
        print '[MyTubeFeedEntry] getUserVideos()'
        username = self.getUserId()
        myuri = 'http://gdata.youtube.com/feeds/api/users/%s/uploads' % username
        print 'Found Uservideos: ', myuri
        return myuri


class MyTubePlayerService():
    cached_auth_request = {}
    current_auth_token = None
    yt_service = None

    def __init__(self):
        print '[MyTube] MyTubePlayerService - init'
        self.feedentries = []
        self.feed = None
        return

    def startService(self):
        print '[MyTube] MyTubePlayerService - startService'
        self.yt_service = gdata.youtube.service.YouTubeService()
        self.yt_service.ssl = self.supportsSSL()
        self.yt_service.developer_key = 'AI39si4AjyvU8GoJGncYzmqMCwelUnqjEMWTFCcUtK-VUzvWygvwPO-sadNwW5tNj9DDCHju3nnJEPvFy4WZZ6hzFYCx8rJ6Mw'
        self.yt_service.client_id = 'ytapi-dream-MyTubePlayer-i0kqrebg-0'
        if self.current_auth_token is not None:
            print '[MyTube] MyTubePlayerService - auth_cached'
            self.yt_service.SetClientLoginToken(self.current_auth_token)
        return

    def stopService(self):
        print '[MyTube] MyTubePlayerService - stopService'
        del self.ytService

    def getLoginTokenOnCurl(self, email, pw):
        opts = {'service': 'youtube',
         'accountType': 'HOSTED_OR_GOOGLE',
         'Email': email,
         'Passwd': pw,
         'source': self.yt_service.client_id}
        print '[MyTube] MyTubePlayerService - Starting external curl auth request'
        result = os.popen('curl -s -k -X POST "%s" -d "%s"' % (gdata.youtube.service.YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL, urlencode(opts))).read()
        return result

    def supportsSSL(self):
        return 'HTTPSConnection' in dir(httplib)

    def getFormattedTokenRequest(self, email, pw):
        return dict(parse_qsl(self.getLoginTokenOnCurl(email, pw).strip().replace('\n', '&')))

    def getAuthedUsername(self):
        if self.cached_auth_request.get('YouTubeUser') is not None:
            return self.cached_auth_request.get('YouTubeUser')
        elif self.is_auth() is False:
            return ''
        else:
            return 'Logged In'

    def auth_user(self, username, password):
        print '[MyTube] MyTubePlayerService - auth_use - ' + str(username)
        if self.yt_service is None:
            self.startService()
        if self.current_auth_token is not None:
            print '[MyTube] MyTubePlayerService - auth_cached'
            self.yt_service.SetClientLoginToken(self.current_auth_token)
            return
        else:
            if self.supportsSSL() is False:
                print '[MyTube] MyTubePlayerService - HTTPSConnection not found trying external curl'
                self.cached_auth_request = self.getFormattedTokenRequest(username, password)
                if self.cached_auth_request.get('Auth') is None:
                    raise Exception('Got no auth token from curl; you need curl and valid youtube login data')
                self.yt_service.SetClientLoginToken(self.cached_auth_request.get('Auth'))
            else:
                print '[MyTube] MyTubePlayerService - Using regularly ProgrammaticLogin for login'
                self.yt_service.email = username
                self.yt_service.password = password
                self.yt_service.ProgrammaticLogin()
            if self.is_auth() is False:
                print '[MyTube] MyTubePlayerService - auth_use - auth not possible resetting'
                self.resetAuthState()
                return
            print '[MyTube] MyTubePlayerService - Got successful login'
            self.current_auth_token = self.auth_token()
            return

    def resetAuthState(self):
        print '[MyTube] MyTubePlayerService - resetting auth'
        self.cached_auth_request = {}
        self.current_auth_token = None
        if self.yt_service is None:
            return
        else:
            self.yt_service.current_token = None
            self.yt_service.token_store.remove_all_tokens()
            return

    def is_auth(self):
        if self.current_auth_token is not None:
            return True
        elif self.yt_service.current_token is None:
            return False
        else:
            return self.yt_service.current_token.get_token_string() != 'None'

    def auth_token(self):
        return self.yt_service.current_token.get_token_string()

    def getFeedService(self, feedname):
        if feedname == 'top_rated':
            return self.yt_service.GetTopRatedVideoFeed
        if feedname == 'most_viewed':
            return self.yt_service.GetMostViewedVideoFeed
        if feedname == 'recently_featured':
            return self.yt_service.GetRecentlyFeaturedVideoFeed
        if feedname == 'top_favorites':
            return self.yt_service.GetTopFavoritesVideoFeed
        if feedname == 'most_recent':
            return self.yt_service.GetMostRecentVideoFeed
        if feedname == 'most_discussed':
            return self.yt_service.GetMostDiscussedVideoFeed
        if feedname == 'most_linked':
            return self.yt_service.GetMostLinkedVideoFeed
        if feedname == 'most_responded':
            return self.yt_service.GetMostRespondedVideoFeed
        return self.yt_service.GetYouTubeVideoFeed

    def getFeed(self, url, feedname = '', callback = None, errorback = None):
        print '[MyTube] MyTubePlayerService - getFeed:', url, feedname
        self.feedentries = []
        ytservice = self.yt_service.GetYouTubeVideoFeed
        if feedname == 'my_subscriptions':
            url = 'http://gdata.youtube.com/feeds/api/users/default/newsubscriptionvideos'
        elif feedname == 'my_favorites':
            url = 'http://gdata.youtube.com/feeds/api/users/default/favorites'
        elif feedname == 'my_history':
            url = 'http://gdata.youtube.com/feeds/api/users/default/watch_history?v=2'
        elif feedname == 'my_recommendations':
            url = 'http://gdata.youtube.com/feeds/api/users/default/recommendations?v=2'
        elif feedname == 'my_watch_later':
            url = 'http://gdata.youtube.com/feeds/api/users/default/watch_later?v=2'
        elif feedname == 'my_uploads':
            url = 'http://gdata.youtube.com/feeds/api/users/default/uploads'
        elif feedname in ('hd', 'most_popular', 'most_shared', 'on_the_web'):
            if feedname == 'hd':
                url = 'http://gdata.youtube.com/feeds/api/videos/-/HD'
            else:
                url = url + feedname
        elif feedname in ('top_rated', 'most_viewed', 'recently_featured', 'top_favorites', 'most_recent', 'most_discussed', 'most_linked', 'most_responded'):
            url = None
            ytservice = self.getFeedService(feedname)
        queryThread = YoutubeQueryThread(ytservice, url, self.gotFeed, self.gotFeedError, callback, errorback)
        queryThread.start()
        return queryThread

    def search(self, searchTerms, startIndex = 1, maxResults = 25, orderby = 'relevance', time = 'all_time', racy = 'include', author = '', lr = '', categories = '', sortOrder = 'ascending', callback = None, errorback = None):
        print '[MyTube] MyTubePlayerService - search()'
        self.feedentries = []
        query = gdata.youtube.service.YouTubeVideoQuery()
        query.vq = searchTerms
        query.orderby = orderby
        query.time = time
        query.racy = racy
        query.sortorder = sortOrder
        if lr is not None:
            query.lr = lr
        if categories[0] is not None:
            query.categories = categories
        query.start_index = startIndex
        query.max_results = maxResults
        queryThread = YoutubeQueryThread(self.yt_service.YouTubeQuery, query, self.gotFeed, self.gotFeedError, callback, errorback)
        queryThread.start()
        return queryThread

    def getUserEntry(self, user = 'default'):
        feed = self.yt_service.GetYouTubeUserEntry(username=user)
        username = feed.username.text
        img_url = str(feed).split('thumbnail')[1].split('"')[1]
        view_count = 0
        if 'totalUploadViews' in str(feed):
            view_count = str(feed).split('totalUploadViews')[1].split('"')[1]
        video_watch_count = feed.statistics.video_watch_count or 0
        subscriber_count = feed.statistics.subscriber_count or 0
        favorite_count = feed.statistics.favorite_count or 0
        user_subscription = 0
        user_liveevent = 0
        user_favorites = 0
        user_contacts = 0
        user_uploads = 0
        temp = str(feed).split('feedLink')
        for entry in temp:
            if 'user.subscriptions' in entry:
                user_subscription = int(entry.split('countHint')[1].split('"')[1])
            if 'user.liveevent' in entry:
                user_liveevent = int(entry.split('countHint')[1].split('"')[1])
            if 'user.favorites' in entry:
                user_favorites = int(entry.split('countHint')[1].split('"')[1])
            if 'user.contacts' in entry:
                user_contacts = int(entry.split('countHint')[1].split('"')[1])
            if 'user.uploads' in entry:
                user_uploads = int(entry.split('countHint')[1].split('"')[1])

        return (username,
         img_url,
         view_count,
         video_watch_count,
         subscriber_count,
         favorite_count,
         user_subscription,
         user_liveevent,
         user_favorites,
         user_contacts,
         user_uploads)

    def gotFeed(self, feed, callback):
        if feed is not None:
            self.feed = feed
            for entry in self.feed.entry:
                MyFeedEntry = MyTubeFeedEntry(self, entry)
                self.feedentries.append(MyFeedEntry)

        if callback is not None:
            callback(self.feed)
        return

    def gotFeedError(self, exception, errorback):
        if errorback is not None:
            errorback(exception)
        return

    def SubscribeToUser(self, username):
        try:
            new_subscription = self.yt_service.AddSubscriptionToChannel(username_to_subscribe_to=username)
            if isinstance(new_subscription, gdata.youtube.YouTubeSubscriptionEntry):
                print '[MyTube] MyTubePlayerService: New subscription added'
                return _('New subscription added')
            return _('Unknown error')
        except gdata.service.RequestError as req:
            return str('Error: ' + str(req[0]['body']))
        except Exception as e:
            return str('Error: ' + e)

    def addToFavorites(self, video_id):
        try:
            video_entry = self.yt_service.GetYouTubeVideoEntry(video_id=video_id)
            response = self.yt_service.AddVideoEntryToFavorites(video_entry)
            if isinstance(response, gdata.youtube.YouTubeVideoEntry):
                print '[MyTube] MyTubePlayerService: Video successfully added to favorites'
                return _('Video successfully added to favorites')
            return _('Unknown error')
        except gdata.service.RequestError as req:
            return str('Error: ' + str(req[0]['body']))
        except Exception as e:
            return str('Error: ' + e)

    def getTitle(self):
        return self.feed.title.text

    def getEntries(self):
        return self.feedentries

    def itemCount(self):
        return self.feed.items_per_page.text

    def getTotalResults(self):
        if self.feed.total_results is None:
            return 0
        else:
            return self.feed.total_results.text

    def getNextFeedEntriesURL(self):
        for link in self.feed.link:
            if link.rel == 'next':
                return link.href

        return None

    def getCurrentPage(self):
        if self.feed.start_index is None:
            return 1
        else:
            return int(int(self.feed.start_index.text) / int(self.itemCount())) + 1


class YoutubeQueryThread(Thread):

    def __init__(self, query, param, gotFeed, gotFeedError, callback, errorback):
        Thread.__init__(self)
        self.messagePump = ePythonMessagePump()
        self.messages = ThreadQueue()
        self.gotFeed = gotFeed
        self.gotFeedError = gotFeedError
        self.callback = callback
        self.errorback = errorback
        self.query = query
        self.param = param
        self.canceled = False
        self.messagePump.recv_msg.get().append(self.finished)

    def cancel(self):
        self.canceled = True

    def run(self):
        try:
            if self.param is None:
                feed = self.query()
            else:
                feed = self.query(self.param)
            self.messages.push((True, feed, self.callback))
            self.messagePump.send(0)
        except Exception as ex:
            self.messages.push((False, ex, self.errorback))
            self.messagePump.send(0)

        return

    def finished(self, val):
        if not self.canceled:
            message = self.messages.pop()
            if message[0]:
                self.gotFeed(message[1], message[2])
            else:
                self.gotFeedError(message[1], message[2])


myTubeService = MyTubePlayerService()